var selectedMarker = null;
var selectedMarkerColor = "red";
var previousLayer = null;
var previousLayerColor = null;
var activeLayerColor = 'red';
var normalMarkerRadius = 5;
var selectedMarkerRadius = 15;
var activeMarkerRadius = 7;

var ligIcon = L.icon({
    iconUrl: 'images/markerSTeamer_27_43.png',
    iconSize: [27, 43], // size of the icon
    iconAnchor: [14, 43] // point of the icon which will correspond
});

function resetLayer(layer) {
    if (layer) {
        layer.eachLayer(function(m) {
            m.setRadius(normalMarkerRadius);
            m.setStyle({fillColor: layer._markersColor, fillOpacity: 0.5});
        });
    }
    previousLayer = null;
}
function selectLayer(layer) {

    // si le layer courant
    // n'est pas le même que le layer du marqueur précédent

    // tous les marqueurs du layer sont mis en rouge 
    layer.eachLayer(function(m) {
        m.setRadius(activeMarkerRadius);
        m.setStyle({fillColor: activeLayerColor, fillOpacity: 1});
    });

    // tous les marqueurs du layer original reprenne leur état initial
    resetLayer(previousLayer);
    // le layercourant est mis a jour avec le layer du marqueur
    previousLayer = layer;

    // le layercourant  est mis au premier plan
    layer.bringToFront();
}

function centerMapOnProject(map, projectInfo) {
    map.fitBounds([
        [projectInfo.minLat - 0.1, projectInfo.minLng - 0.1],
        [projectInfo.maxLat + 0.1, projectInfo.maxLng + 0.1]
    ]);
}
function displayProjectInfo(infoProject) {
    var htmlProjectDescription = '<h3>' + infoProject.project + '</h3>' + infoProject.description;
    if (infoProject["web site"]) {
        htmlProjectDescription = htmlProjectDescription + '<br><a href="' + infoProject["web site"] + '">' + infoProject["web site"] + '</a>';
    }
    $("#projectDiv").html(htmlProjectDescription);
}

function createProjectLayer(projectName, projectsData, partnersData, couleur, centerLat, centerLong, zoomLevel) {
    var layer = new L.FeatureGroup();
    var i;
    var marker;
    var projInfo = null;

    var nbPartners = 0;

    layer._markersColor = couleur;

    for (i = 0; i < projectsData.length; i++) {
        if (projectsData[i].project === projectName) {
            projInfo = projectsData[i];
        }
    }
    projInfo.minLat = 45.1940330;    // center latitude of project ,
    projInfo.minLng = 5.767818600000055;    // center longitude of project
    projInfo.maxLat = 45.194033;
    projInfo.maxLng = 5.76781860000005;
    for (i = 0; i < partnersData.length; i++) {
        //if (partnersData[i].projects === projectName) {
        var pprojects = partnersData[i].projects;
        if ((pprojects.indexOf(projectName)) > -1) {
            if (partnersData[i].lat < projInfo.minLat) {
                projInfo.minLat = partnersData[i].lat;
            }
            if (partnersData[i].lat > projInfo.maxLat) {
                projInfo.maxLat = partnersData[i].lat;
            }
            if (partnersData[i].lng < projInfo.minLng) {
                projInfo.minLng = partnersData[i].lng;
            }
            if (partnersData[i].lng > projInfo.maxLng) {
                projInfo.maxLng = partnersData[i].lng;
            }
            projInfo.lat += partnersData[i].lat;
            projInfo.lng += partnersData[i].lng;
            nbPartners++;
            marker = L.circleMarker([partnersData[i].lat, partnersData[i].lng], {color: couleur, fillColor: couleur, fillOpacity: 0.5}); // ,10, {color: 'red', fillColor: 'red', fillOpacity: 0.5} L.marker([partnersData[i].lat, partnersData[i].lng]);
            marker.setRadius(normalMarkerRadius);
            marker.addTo(layer);
            partnersData[i].couleur = couleur;
            marker.info = partnersData[i];
            marker.infoProject = projInfo;
            //  marker.originLayer = layer;

            marker.on('mouseover', function(e) {
                if (previousLayer !== layer) {
                    selectLayer(layer);
                }
                // sinon
                else {
                    if (selectedMarker) {
                        selectedMarker.setRadius(activeMarkerRadius);
                        selectedMarker.setStyle({fillColor: activeLayerColor, fillOpacity: 1});
                    }
                }
                selectedMarker = this;
                selectedMarkerColor = this.options.fillColor;
                $("#partnerDiv").html(this.info.partner + '<br><a href="' + this.info["web site"] + '" target="_blank">' + this.info["web site"] + '</a>');

                // layer.bringToFront();
                displayProjectInfo(this.infoProject);


                this.setStyle({fillColor: 'yellow'});
                this.setRadius(selectedMarkerRadius);
            }
            );
            marker.on('mouseout', function(e) {
                //   this.setRadius(normalMarkerRadius);
            });
        }


    }
    layer.StyledLayerControl = {
        legendColor: couleur,
        layerZoom: zoomLevel,
        projectInfo: projInfo
    };
    return layer;
}

/**
 * Creation of a layers group for a given project level.
 * For exemple the layer group for all the International projects
 * @param {String} level
 * @param {String} levelcolor
 * @param {String} levelzoom
 * @returns {String} 
 */
function createLevelProjects(level, levelcolor, levelzoom) {
    var projectsList = " ";
    for (i = 0; i < projects.length; i++) {
        if (projects[i].level === level) {
            projectsList = projectsList + ' "' + projects[i].project + '" : ' + 'createProjectLayer( "' + projects[i].project + '", projects, partners, ' + levelcolor + ', 45.19, 5.77,' + levelzoom + ')' + ',';
        }
    }
    return projectsList;
}

function init() {
    // Google layers
    var g_roadmap = new L.Google('ROADMAP');
    var g_satellite = new L.Google('SATELLITE');
    var g_terrain = new L.Google('TERRAIN');

    // OSM layers
    var osmUrl = 'http://{s}.tile.osm.org/{z}/{x}/{y}.png';
    var osmAttrib = 'Map data © <a href="http://openstreetmap.org">OpenStreetMap</a> contributors';
    var osm = new L.TileLayer(osmUrl, {attribution: osmAttrib});

    // Bing layers
    var bing1 = new L.BingLayer("AvZ2Z8Jve41V_bnPTe2mw4Xi8YWTyj2eT87tSGSsezrYWiyaj0ldMaVdkyf8aik6", {type: 'Aerial'});
    var bing2 = new L.BingLayer("AvZ2Z8Jve41V_bnPTe2mw4Xi8YWTyj2eT87tSGSsezrYWiyaj0ldMaVdkyf8aik6", {type: 'Road'});


    var map = L.map('map', {
        center: [45.194033, 5.767818600000055],
        zoom: 6
    });

    map.addLayer(osm);

    var baseMaps = [
        {
            groupName: "Base Maps",
            expanded: false,
            layers: {
                "OpenStreetMaps": osm,
                "Google Satellite": g_satellite,
                "Google Road Map": g_roadmap,
                "Google Terreno": g_terrain,
                "Bing Satellite": bing1,
                "Bing Road": bing2
            }

        }
    ];

    var overlayers = "[ "
            + "{groupName: " + '"International Projects"' + ", expanded: false, layers: { " + createLevelProjects("International", '"green"', '2') + "}},"
            + "{groupName: " + '"National Projects"' + ", expanded: false, layers: { " + createLevelProjects("National", '"blue"', '5') + "}},"
            + "{groupName: " + '"Local Projects"' + ", expanded: false, layers: { " + createLevelProjects("Local", '"orange"', '10') + "}},"
            + "];";

    var overlays = eval(overlayers);

    var options = {
        container_width: "300px",
        group_maxHeight: "80px",
        //container_maxHeight : "350px", 
        exclusive: false
    };

    var control = L.Control.styledLayerControl(baseMaps, overlays, options);
    map.addControl(control);

    L.marker([45.194033, 5.767818600000055], {icon: ligIcon}).addTo(map);
}